<?php

/**
  *
  *  @file
  *    Helper functions for the Duo module.  Includes the Duo API
  *    to reduce the number of files.
  */

/**
  *  Helper function to check if somebody is logged in already
  *
  *  Checks to see if a user is logged in, if they are, it'll redirect them and set an error message
  *
  *  @return void
  */
  function _check_login() {
    if (user_is_logged_in()) {
      drupal_set_message( t('You are already logged in.'), 'error');
      drupal_goto();
    }
  }

/**
  *  HTML Generation Functions
  *
  *  The following functions will all generate an HTML page to be displayed before loggin the user in.
  *
  *  @see _duo_generate_header()
  *  @see _duo_generate_form()
  *  @see _duo_generate_footer()
  *  @see _duo_generate_help()
  */


  /**
    *  Generate Duo Login page header.
    *
    *  @return HTML for the Duo page header.
    */
  function  _duo_generate_header() {

    // Generate the HTML head.
    $header  = '<html>';
    $header .= '<head>';
    $header .= '<link rel="stylesheet" type="text/css" href="' . base_path() . drupal_get_path('module', 'duo') . '/resources/duo.css" >';
    $header .= '<link rel="stylesheet" type="text/css" href="' . base_path() . drupal_get_path('module', 'duo') . '/resources/960.css" >';
    $header .= '<link rel="stylesheet" type="text/css" href="' . base_path() . drupal_get_path('module', 'duo') . '/resources/custom.css" >';
    $header .= '</head>';
    $header .= '<body>';

    // Create the Duo Header.
    $header .= '<div id="header">';
    $header .= '<div class="container_12">';
    $header .= '<div class="grid_5">';

    // Link to Duo in new tab so end users can understand who Duo is and why they can trust them.
    $header .= '<a href="http://www.duosecurity.com" target="_new">';
    $header .= '<img src="' . base_path() . drupal_get_path('module', 'duo') . '/resources/logo-small.png" ></a>';
    $header .= '<h1 class="logo"><a href="http://duosecurity.com">Duo Security</a></h1>';
    $header .= '</div>';
    $header .= '</div>';
    $header .= '<div class="clear"></div>';
    $header .= '</div>';

    // End header bar, start the page.
    $header .= '<br /><h1 id="center">Login with Duo</h1>';

    return $header;
  }


/**
  *  Generates Duo Login Form.
  *
  *  @param $namehash
  *    $namehash is the hashed username, expiration time and uid
  *
  *  @param $time
  *    $time is the expiration timestamp
  *
  *  @param $uid
  *    $uid is the user_id of the person logging in
  *
  *  @param $sig_req
  *    $sig_req is the signature that duo requires to be posted to their servers
  *
  *  @return HTML for the duo login iframe/form
  */
  function _duo_generate_form($namehash, $time, $uid, $sig_req) {

    $apiurl = variable_get('duo_apiurl', 'api-eval.duosecurity.com');

    $form  = '<script src="' . base_path() . drupal_get_path('module', 'duo') . '/duo.js" ></script>';
    $form .= "<script>Duo.init({'host':'" . $apiurl . "','post_action':'" . url("duo_login_process", array("absolute" => TRUE)) . "','sig_request':'" . $sig_req . "'});";
    $form .= '</script>';

    $form .= '<div id="centered">';
    $form .= '<iframe id="duo_iframe" width="100%" height="500" frameborder="0"></iframe>';
    $form .= '</div>';

    // Hidden form with required material to verify user credentials:
    $form .= '<form method="POST" id="duo_form">';
    $form .= '<input type="hidden" name="name" value="' . $namehash . '"/>';
    $form .= '<input type="hidden" name="time" value="' . $time . '"/>';
    $form .= '<input type="hidden" name="uid" value="' . $uid . '" />';
    $form .= '</form>';

    return $form;
  }

  /**
    *  Generates HTML Footer.
    *
    *  @return HTML footer that closes the page
    */
  function  _duo_generate_footer() {
    $footer  = '</body>';
    $footer .= '</html>';
    return $footer;
  }

  /**
    *  Generates Help Text.
    *
    *  This is just to remove all HTML from the module itself.
    *
    *  @return Help HTML
    */
  function _duo_generate_help() {
    $help = '<h1>' . t("Duo Two Factor Authentication") . '</h1>';

    $help .= '<h2>' . t("About") . '</h2>';
    $help .= '<p>' . t("This plugin adds another layer of security to the login method by implementing Duo's two factor authentication") . '</p>';

    $help .= '<p>' . t("Duo's two factor authentication works by forcing a user to use their phone to login in addition to their username and password.  After logging in, your users will be asked to register their phones with Duo, upon registering, they will be able to login in using Duo's authentication system.") . '</p>';


    $help .= '<h2>' . t("Setting up") . '</h2>';
    $help .= '<p>' . t("To set up this plugin, you need to sign up with an account at") . ' <a href="http://www.duosecurity.com" target="_blank">duosecurity.com</a>.</p>';

    $help .= '<p>' . t("After signing up with Duo, you will be provided with an integration key, and secret key, as well as an API URL.  Once you have received those keys and url, you must configure this Drupal plugin.  If this plugin is left unconfigured, users will continue to login as usual, without the second factor authentication.") . '</p>';

    $help .= '<p>' . t("Lastly, clear the cache.  If you don't clear the cache, Duo won't work.") . '</p>';
    return $help;
  }


/**
  *  Duo's PHP API
  *
  *  Signs and decodes a signed response.
  *
  */
class Duo {
  const DUO_PREFIX = "TX";
  const APP_PREFIX = "APP";
  const AUTH_PREFIX = "AUTH";

  const DUO_EXPIRE = 300;
  const APP_EXPIRE = 3600;

  const IKEY_LEN = 20;
  const SKEY_LEN = 40;
  const AKEY_LEN = 40; // if this changes you have to change ERR_AKEY

  const ERR_USER = 'ERR|The username passed to sign_request() is invalid.';
  const ERR_IKEY = 'ERR|The Duo integration key passed to sign_request() is invalid.';
  const ERR_SKEY = 'ERR|The Duo secret key passed to sign_request() is invalid.';
  const ERR_AKEY = "ERR|The application secret key passed to sign_request() must be at least 40 characters.";

  private static function sign_vals($key, $vals, $prefix, $expire) {
    $exp = time() + $expire;

    $val = $vals . '|' . $exp;
    $b64 = base64_encode($val);
    $cookie = $prefix . '|' . $b64;

    $sig = hash_hmac("sha1", $cookie, $key);
    return $cookie . '|' . $sig;
  }

  private static function parse_vals($key, $val, $prefix) {
    $ts = time();
    list($u_prefix, $u_b64, $u_sig) = explode('|', $val);

    $sig = hash_hmac("sha1", $u_prefix . '|' . $u_b64, $key);
    if (hash_hmac("sha1", $sig, $key) != hash_hmac("sha1", $u_sig, $key)) {
      return NULL;
    }

    if ($u_prefix != $prefix) {
      return NULL;
    }

  list($user, $ikey, $exp) = explode('|', base64_decode($u_b64));

    if ($ts >= intval($exp)) {
      return NULL;
    }

    return $user;
  }

  public static function signRequest($ikey, $skey, $akey, $username) {
    if (!isset($username) || drupal_strlen($username) == 0) {
      return self::ERR_USER;
    }
    if (!isset($ikey) || drupal_strlen($ikey) != self::IKEY_LEN) {
      return self::ERR_IKEY;
    }
    if (!isset($skey) || drupal_strlen($skey) != self::SKEY_LEN) {
      return self::ERR_SKEY;
    }
    if (!isset($akey) || drupal_strlen($akey) < self::AKEY_LEN) {
      return self::ERR_AKEY;
    }

    $vals = $username . '|' . $ikey;

    $duo_sig = self::sign_vals($skey, $vals, self::DUO_PREFIX, self::DUO_EXPIRE);
    $app_sig = self::sign_vals($akey, $vals, self::APP_PREFIX, self::APP_EXPIRE);

    return $duo_sig . ':' . $app_sig;
  }

  public static function verifyResponse($ikey, $skey, $akey, $sig_response) {
    list($auth_sig, $app_sig) = explode(':', $sig_response);

    $auth_user = self::parse_vals($skey, $auth_sig, self::AUTH_PREFIX);
    $app_user = self::parse_vals($akey, $app_sig, self::APP_PREFIX);

    if ($auth_user != $app_user) {
      return NULL;
    }

    return $auth_user;
  }
}
